/**
 * Documentation for itemRegister.js
 *
 * This file contains the definition and registration of various items in the form of an array of objects.
 * Each object within the array represents a different item and includes specific properties related to that item.
 *
 * Structure of Data:
 * - itemRegister: An array containing objects with the following properties:
 *   - item: The name of the item as a string.
 *   - specific: An array of objects that define specific characteristics of the item, such as type and duration.
 * 
 * Details of the 'item' Property:
 * - Must be in single string indicating prefix of the item typeId.
 * 
 * Details of the 'specific' Property:
 * - type: Can be a single string or an array of strings indicating the type or suffix of the item typeId.
 *   - Single string only supported "all" value.
 * - duration: A numeric value representing the duration or other time-related characteristic associated with the item.
 *   - Allowed value is a Number.
 */

const itemRegister = [
    {
        item: "sword",
        specific: [
            {
                type: "all",
                duration: 0.6
            }
        ],
    },
    {
        item: "_axe",
        specific: [
            {
                type: ["wooden", "stone"],
                duration: 1.25
            },
            {
                type: ["diamond", "gold", "netherite"],
                duration: 1
            },
            {
                type: ["iron"],
                duration: 1.1
            }
        ]
    },
    {
        item: "trident",
        specific: [
            {
                type: "all",
                duration: 0.9
            }
        ]
    },
    {
        item: "pickaxe",
        specific: [
            {
                type: "all",
                duration: 0.85
            }
        ]
    },
    {
        item: "shovel",
        specific: [
            {
                type: "all",
                duration: 1
            }
        ]
    },
    {
        item: "hoe",
        specific: [
            {
                type: ["wooden", "gold"],
                duration: 1
            },
            {
                type: ["stone"],
                duration: 0.5
            },
            {
                type: ["iron"],
                duration: 0.35
            },
            {
                type: ["diamond", "netherite"],
                duration: 0.25
            }
        ]
    }
]

export {
    itemRegister
}