import { system, world } from "@minecraft/server";
import { itemInfo } from "./itemHandler";

export let playerdata = {};

system.runInterval(() => {
  for (const player of world.getPlayers()) {
    let sprinting = player.isSprinting;
    let onground = player.isOnGround;

    system.runTimeout(() => {
      playerdata[player.id] = {
        isSprinting: sprinting,
        isOnGround: onground,
      };
    }, 1);
  }
});

// Manage cooldown timers in an interval
system.runInterval(() => {
  world.getPlayers().forEach(player => {
    let sweepTimer = player.getDynamicProperty("cd:sweepTimer") || 0;
    if (sweepTimer > 0) {
      sweepTimer -= 0.05;
      player.setDynamicProperty("cd:sweepTimer", sweepTimer);
    }

    let knockTimer = player.getDynamicProperty("cd:knock") || 0;
    if (knockTimer > 0) {
      knockTimer -= 0.05;
      player.setDynamicProperty("cd:knock", knockTimer);
    }
  });
}, 1); // Run this check every tick (0.05 seconds)

const knockbackResistanceMap = new Map();

world.afterEvents.entityHitEntity.subscribe((event) => {
  const player = event.damagingEntity;
  if (player?.typeId !== "minecraft:player") return;

  const inv = player.getComponent("inventory")?.container;
  const item = inv?.getItem(player.selectedSlotIndex);

  // Sweep attack cooldown logic
  let sweepTimer = player.getDynamicProperty("cd:sweepTimer") || 0;
  const sweepCooldownDuration = 0.50; // Duration of cooldown for sweep attacks

  if (sweepTimer > 0) {
    console.warn(`Player ${player.id} is still on sweep cooldown: ${sweepTimer}`);
    return; // Exit if still in sweep cooldown
  }

  if (item && item.typeId.includes("sword") && playerdata[player.id]?.isOnGround && !playerdata[player.id]?.isSprinting) {
    const dimension = world.getDimension(player.dimension.id);

    // Calculate the position in front of the player
    const direction = player.getRotation().y * (Math.PI / 180);
    const offsetX = -Math.sin(direction) * 2; // Adjust the distance in front of the player
    const offsetZ = Math.cos(direction) * 2;
    const sweepPosition = {
      x: player.location.x + offsetX,
      y: player.location.y + 0.2, // Adjust the Y position slightly above the ground
      z: player.location.z + offsetZ,
    };

    // Apply the sweep attack
    dimension.spawnParticle("dave:sweep", sweepPosition);

world.playSound("sweep", sweepPosition);

dimension.getEntities({
  location: event.hitEntity.location,
  maxDistance: 1.5,
}).forEach(entity => {
  if (entity.id === player.id || entity.id === event.hitEntity.id) return;

  // Apply damage with a random multiplier between 0.50 and 0.75
  const damage = 3 * (Math.random() * (0.75 - 0.50) + 0.50);
  entity.applyDamage(damage, {
    cause: "entityAttack",
    damagingEntity: player,
  });
});


    // Start the cooldown for the sweep attack
    player.setDynamicProperty("cd:sweepTimer", sweepCooldownDuration);
    console.warn(`Player ${player.id} started sweep cooldown.`);
  }

  // Knockback logic with custom knockback resistance
  let knockTimer = player.getDynamicProperty("cd:knock") || 0;
  if (knockTimer == 0 && playerdata[player.id]?.isSprinting && player?.isOnGround) {
    const hitEntity = event.hitEntity;

    // Check if the entity has knockback resistance
    if (knockbackResistanceMap.has(hitEntity.id)) {
      console.warn(`Entity ${hitEntity.id} resisted knockback due to recent hit.`);
    } else {
      // Apply knockback
      const verticalKnockback = 0.35; // Standard vertical knockback
      hitEntity.applyKnockback(
        -Math.sin(player.getRotation().y * (Math.PI / 180)),
        Math.cos(player.getRotation().y * (Math.PI / 180)),
        1.1, // Horizontal
        verticalKnockback // Vertical knockback
      );

      // Apply custom knockback resistance
      knockbackResistanceMap.set(hitEntity.id, true);
      system.runTimeout(() => {
        knockbackResistanceMap.delete(hitEntity.id); // Remove knockback resistance after 1 second
      }, 1); // 20 ticks = 1 second
    }

    player.setDynamicProperty("cd:knock", 1); // Start knockback cooldown

    // Knockback sound effect
    world.playSound("knockback", event.hitEntity.location);

    let runKnock = system.runInterval(() => {
      knockTimer++;
      player.setDynamicProperty("cd:knock", knockTimer);
      if (knockTimer > 5) {
        player.setDynamicProperty("cd:knock", 0);
        system.clearRun(runKnock);
      }
    }, 1);
  }
});