import { world, system, TicksPerSecond } from "@minecraft/server";

// Golden Apple absorption reset
world.afterEvents.itemCompleteUse.subscribe(use => {
    try {
        let p = use.source;
        let item = use.itemStack;

        if (item.typeId === "minecraft:golden_apple") {
            p.runCommandAsync("effect @s absorption 0 0");
            p.runCommandAsync("effect @s absorption 120 0 true");
        }

    } catch (e) {
        console.warn("Error handling golden apple effect:", e);
    }
});

const tps = TicksPerSecond;
const foods = [
	{ item: "minecraft:raw_beef", duration: 2 },
	{ item: "minecraft:raw_chicken", duration: 2 },
	{ item: "minecraft:raw_cod", duration: 1 },
	{ item: "minecraft:raw_mutton", duration: 2 },
	{ item: "minecraft:raw_porkchop", duration: 2 },
	{ item: "minecraft:raw_rabbit", duration: 2 },
	{ item: "minecraft:raw_salmon", duration: 1 },
	{ item: "minecraft:rotten_flesh", duration: 2 },
    { item: "minecraft:cooked_beef", duration: 10 },
	{ item: "minecraft:glow_berries", duration: 1 },
	{ item: "minecraft:sweet_berries", duration: 2 },
	{ item: "minecraft:honey_bottle", duration: 2 },
	{ item: "minecraft:potato", duration: 1 },
	{ item: "minecraft:pumpkin_pie", duration: 3 },
	{ item: "minecraft:baked_potato", duration: 3 },
	{ item: "minecraft:beetroot", duration: 2 },
	{ item: "minecraft:chorus_fruit", duration: 2 },
    { item: "minecraft:cooked_chicken", duration: 7 },
    { item: "minecraft:cooked_cod", duration: 5 },
    { item: "minecraft:cooked_mutton", duration: 9 },
    { item: "minecraft:cooked_porkchop", duration: 13 },
    { item: "minecraft:cooked_rabbit", duration: 9 },
    { item: "minecraft:cooked_salmon", duration: 7 },
    { item: "minecraft:bread", duration: 5 },
    { item: "minecraft:mushroom_stew", duration: 12 },
    { item: "minecraft:suspicious_stew", duration: 20 },
    { item: "minecraft:rabbit_stew", duration: 12 },
    { item: "minecraft:beetroot_soup", duration: 12 },
    { item: "minecraft:baked_potato", duration: 12 },
    { item: "minecraft:cookie", duration: 6 },
    { item: "minecraft:pumpkin_pie", duration: 12 },
    { item: "minecraft:apple", duration: 9 },
    { item: "minecraft:golden_carrot", duration: 12 },
	{ item: "minecraft:carrot", duration: 4 }, 
	{ item: "minecraft:melon_slice", duration: 2 },
    { item: "minecraft:golden_apple", duration: 9 },
    { item: "minecraft:enchanted_golden_apple", duration: 18 },
];

world.afterEvents.itemCompleteUse.subscribe(({ itemStack, source }) => {
    let player = source;

    // Initialize dynamic properties if they are undefined
    let saturationTime = Math.min(player.getDynamicProperty("saturationTime") || 0, 20);
    let saturationCount = Math.min(player.getDynamicProperty("saturationCount") || 0, 20);

    foods.forEach(food => {
        if (itemStack.typeId === food.item) {
            if (saturationTime === 0) {
                console.warn(food.item);
                player.addTag("dave:onSaturationEffect");
                saturationTime += food.duration;
                saturationTime = Math.min(saturationTime, 20); // Cap at 20
                player.setDynamicProperty("saturationTime", saturationTime);
                console.warn("passed food eaten = 0");
            } else if (saturationTime > 0) {
                console.warn(food.item);
                player.addTag("dave:onSaturationEffect");
                saturationTime += food.duration / (food.duration / 2);
                saturationTime = Math.min(saturationTime, 20); // Cap at 20
                player.setDynamicProperty("saturationTime", saturationTime);
                console.warn("passed food eaten > 0");
            }
        }
    });
});

system.runInterval(() => {
    world.getAllPlayers().forEach(player => {
        if (player.hasTag("dave:onSaturationEffect")) {
            let saturationCount = Math.min(player.getDynamicProperty("saturationCount") || 0, 20);
            let saturationTime = Math.min(player.getDynamicProperty("saturationTime") || 0, 20);

            saturationCount += 1;
            saturationCount = Math.min(saturationCount, 20); // Cap at 20
            player.setDynamicProperty("saturationCount", saturationCount);
            console.warn(saturationCount);

            let health = player.getComponent("minecraft:health").currentValue;
            if (health < 20 && saturationCount % 2 === 0) {
                console.warn("regen");
                player.addEffect("regeneration", 1 * tps, { amplifier: 3, showParticles: false });
                player.addEffect("hunger", 1 * tps, { amplifier: 110, showParticles: false });
                //player.runCommandAsync("hud @s hide status_effects");
                system.runTimeout(() => {
                    //player.runCommandAsync("hud @s reset status_effects");
                }, 1 * tps);
            }

            if (saturationCount >= saturationTime) {
                console.warn("Stop");
                player.setDynamicProperty("saturationTime", 0);
                player.setDynamicProperty("saturationCount", 0);
                player.removeTag("dave:onSaturationEffect");
            }
        }
    });
}, 20);

world.afterEvents.playerSpawn.subscribe(({ player }) => {
    if (!player.hasTag("dave:setupFoodSaturations")) {
        player.setDynamicProperty("saturationTime", 0);
        player.setDynamicProperty("saturationCount", 0);
        player.addTag("dave:setupFoodSaturations");
        console.warn("Setup complete");
    }
});